/**
 * Copyright (c) 2011-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package com.ebmwebsourcing.jbi.adaptor.impl.exchange;

import java.net.URI;
import java.util.logging.Logger;

import javax.jbi.messaging.MessageExchange.Role;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.xml.namespace.QName;

import org.ow2.petals.jbi.messaging.exchange.AbstractContextualMessageExchangeFactory;
import org.ow2.petals.jbi.messaging.exchange.ContextualMessageExchangeImplBuilder;
import org.ow2.petals.jbi.messaging.exchange.MessageExchangeImpl;
import org.ow2.petals.jbi.messaging.exchange.MessageExchangeWrapper;

import com.ebmwebsourcing.jbi.adaptor.impl.exchange.types.InOnlyImpl;
import com.ebmwebsourcing.jbi.adaptor.impl.exchange.types.InOptionalOutImpl;
import com.ebmwebsourcing.jbi.adaptor.impl.exchange.types.InOutImpl;
import com.ebmwebsourcing.jbi.adaptor.impl.exchange.types.RobustInOnlyImpl;

/**
 * @author Adrien Ruffie - EBM WebSourcing
 */
public class ContextualMessageExchangeFactoryImpl extends
        AbstractContextualMessageExchangeFactory {

    private ContextualMessageExchangeFactoryImpl(ServiceEndpoint consumerEndpoint,
            ContextualMessageExchangeImplBuilder messageExchangeImplBuilder, Logger logger) {
        super(consumerEndpoint, messageExchangeImplBuilder, logger);
    }

    public static final ContextualMessageExchangeFactoryImpl createContextualMessageExchangeFactory(
            ServiceEndpoint consumerEndpoint, final Logger logger) {
        ContextualMessageExchangeImplBuilder messageExchangeImplBuilder = ContextualMessageExchangeImplBuilder
                .createMessageExchangeImplBuilder(consumerEndpoint);
        return new ContextualMessageExchangeFactoryImpl(consumerEndpoint,
                messageExchangeImplBuilder, logger);
    }

    public static final ContextualMessageExchangeFactoryImpl createContextualMessageExchangeFactoryForEndpoint(
            ServiceEndpoint consumerEndpoint, final Logger logger, ServiceEndpoint targetEndpoint) {
        ContextualMessageExchangeImplBuilder messageExchangeImplBuilder = ContextualMessageExchangeImplBuilder
                .createMessageExchangeImplBuilderForEndpoint(consumerEndpoint, targetEndpoint);
        return new ContextualMessageExchangeFactoryImpl(consumerEndpoint,
                messageExchangeImplBuilder, logger);
    }

    public static final ContextualMessageExchangeFactoryImpl createContextualMessageExchangeFactoryForInterface(
            ServiceEndpoint consumerEndpoint, final Logger logger, QName targetInterface) {
        ContextualMessageExchangeImplBuilder messageExchangeImplBuilder = ContextualMessageExchangeImplBuilder
                .createMessageExchangeImplBuilderForInterface(consumerEndpoint, targetInterface);
        return new ContextualMessageExchangeFactoryImpl(consumerEndpoint,
                messageExchangeImplBuilder, logger);
    }

    public static final ContextualMessageExchangeFactoryImpl createContextualMessageExchangeFactoryForService(
            ServiceEndpoint consumerEndpoint, final Logger logger, QName targetService) {
        ContextualMessageExchangeImplBuilder messageExchangeImplBuilder = ContextualMessageExchangeImplBuilder
                .createMessageExchangeImplBuilderForService(consumerEndpoint, targetService);
        return new ContextualMessageExchangeFactoryImpl(consumerEndpoint,
                messageExchangeImplBuilder, logger);
    }

    @Override
    protected MessageExchangeWrapper createMessageExchangeDecoratorForPattern(
            MessageExchangeImpl exchange, URI pattern) {
        if (isInOnlyPattern(pattern)) {
            return new InOnlyImpl(exchange, Role.CONSUMER);
        } else if (isRobustInOnlyPattern(pattern)) {
            return new RobustInOnlyImpl(exchange, Role.CONSUMER);
        } else if (isInOutPattern(pattern)) {
            return new InOutImpl(exchange, Role.CONSUMER);
        } else if (isInOptionalOutPattern(pattern)) {
            return new InOptionalOutImpl(exchange, Role.CONSUMER);
        } else {
            return new MessageExchangeWrapper(exchange, Role.CONSUMER);
        }
    }

}
